import { normalize } from '@angular-devkit/core';
import { strings } from '@angular-devkit/core';

import { apply, move, Rule, template, url, mergeWith, chain, Tree } from '@angular-devkit/schematics';
import { GeneratorInterface } from '../GeneratorInterface';
import { GeneratorHelper } from '../GeneratorHelper';

const ROOT_REDUCER_FILE_PATH = 'src/app/store/reducers.ts';
const ROOT_SAGA_FILE_PATH = 'src/app/store/sagas.ts';

export class EmptyModuleGenerator implements GeneratorInterface {
  addDeclarationToReducer(options: any): Rule {
    return (host: Tree) => {
      let readVal = host.read(ROOT_REDUCER_FILE_PATH);
      const declarationRecorder = host.beginUpdate(ROOT_REDUCER_FILE_PATH);
      if (readVal) {
        let val = readVal || '';
        if (val) {
          let str = val.toString();
          if (str.indexOf('const RootReducer = (') > -1) {
            declarationRecorder.insertLeft(0, `import { ${strings.classify(options.name)} } from '../components/${options.name}/${options.name}.reducer';\n`);
          }
          if (str.indexOf('combineReducers({') > -1) {
            declarationRecorder.insertRight(str.indexOf('combineReducers({') + 'combineReducers({'.length, `${strings.classify(options.name)}, `);
          }
        }
      }
      host.commitUpdate(declarationRecorder);
    };
  }

  addDeclarationToSagas(options: any): Rule {
    return (host: Tree) => {
      let readVal = host.read(ROOT_SAGA_FILE_PATH);
      const declarationRecorder = host.beginUpdate(ROOT_SAGA_FILE_PATH);
      if (readVal) {
        let val = readVal || '';
        if (val) {
          let str = val.toString();
          if (str.indexOf('function* watchApiCallError() {') > -1) {
            declarationRecorder.insertLeft(0, `import ${strings.camelize(options.name)} from '../components/${options.name}/${options.name}.saga';\n`);
          }
          if (str.indexOf('yield [') > -1) {
            declarationRecorder.insertRight(str.indexOf('yield [') + 'yield ['.length, `...${strings.camelize(options.name)}, `);
          }
        }
      }
      host.commitUpdate(declarationRecorder);
    };
  }

  generate(options: any): Rule {
    options.path = options.path ? normalize(options.path) : options.path;

    const merge = [];
    const declaretion = [];

    let data = {
      ...strings,
      ...options
    };

    data = {
      ...data,
      ...{
        resolveRelativePath: new GeneratorHelper().resolveRelativePath.bind(data),
        generateFilePath: new GeneratorHelper().generateFilePath.bind(data),
        resolveApiModule: new GeneratorHelper().resolveApiModule.bind(data),
        resolveModelName: new GeneratorHelper().resolveModelName.bind(data),
        generateFilePathForApi: new GeneratorHelper().generateFilePathForApi.bind(data)
      }
    };

    merge.push(apply(url('./files/emptymodule/app/components'), [template(data), move(`${options.sourceDir}/app/components`)]));
    if (options.noredux !== true) {
      declaretion.push(this.addDeclarationToReducer(options));
      declaretion.push(this.addDeclarationToSagas(options));
    }
    return chain([...merge.map(item => mergeWith(item)), ...declaretion]);
  }
}
