import { strings } from '@angular-devkit/core';
import * as pluralize from 'pluralize';

export class GeneratorHelper {
  resolveRelativePath(absolute_path: String, current_file_path: String) {
    let diff_absolute_path;

    const splited_absolute_path = absolute_path.split('/');
    const splited_current_file_path = current_file_path.split('/');
    const avg_length = (splited_absolute_path.length + splited_current_file_path.length) / 2;

    let diff_index = 0;
    for (let i = 0; i <= avg_length; i++) {
      if (splited_absolute_path[i] !== splited_current_file_path[i]) {
        break;
      }
      diff_index += splited_absolute_path[i].length + 1;
    }

    diff_absolute_path = absolute_path.slice(diff_index);
    const diff_current_file_path = current_file_path.slice(diff_index);
    const depth_depth_diff_absolute_path = diff_current_file_path.split('/').length;
    let ard = [];
    for (let i = 1; i < depth_depth_diff_absolute_path; i++) {
      ard.push(i);
    }
    return ard.map(() => '..').join('/') + '/' + diff_absolute_path;
  }

  generateFilePath(path: String) {
    const { component } = this as any;
    if (component === undefined) {
      return `src/app/${path}`;
    } else {
      return `src/app/components/${component}/${path}`;
    }
  }

  generateFilePathForApi(path: String) {
    const { apifolderpath } = this as any;
    if (apifolderpath === undefined) {
      return `src/app/${path}`;
    } else {
      return `src/app/${apifolderpath}/${path}`;
    }
  }

  resolveApiModule() {
    const { api, apifolderpath, camelize, name } = this as any;
    if (api === undefined) {
      if (apifolderpath === undefined) {
        return camelize(name);
      } else {
        return `${apifolderpath}.${camelize(name)}`;
      }
    } else {
      return api;
    }
  }

  resolveModelName() {
    const { model, classify, name } = this as any;
    if (model === undefined) {
      return classify(name);
    } else {
      return model;
    }
  }

  actionGenerator(prefix: any, suffix: any): String {
    let { name, component } = this as any;
    if (component !== undefined) {
      return `${strings.underscore(prefix)}_${strings.underscore(component.replace('/', '_'))}_${strings.underscore(name)}_${strings.underscore(suffix)}`.toUpperCase();
    } else {
      return `${strings.underscore(prefix)}_${strings.underscore(name)}_${strings.underscore(suffix)}`.toUpperCase();
    }
  }

  pluralizeName(transformer: Function) {
    const result = pluralize((this as any).name);
    if (transformer) {
      return transformer(result);
    }
    return result;
  }

  reducerGenerator(name: any) {
    if (name === undefined) {
      name = (this as any).name;
    }
    return strings.classify(name);
  }

  fullReducerGenerator(suffix: String) {
    const { component, name } = this as any;
    let result;
    if (component !== undefined) {
      const pre = component
        .split('/')
        .map((i: any) => strings.classify(i))
        .join('.');
      result = `${pre}.${strings.classify(name)}`;
    } else {
      result = strings.classify(name);
    }

    if (suffix !== undefined) {
      result = `${result}.${suffix}`;
    }
    return result;
  }

  routerGenerator(prefix: String) {
    const { component, name } = this as any;
    let result;
    if (component !== undefined) {
      result = strings.classify(`${component}_${name}`);
    } else {
      result = strings.classify(name);
    }

    if (prefix !== undefined) {
      result = `${prefix}${result}`;
    }
    return result;
  }
}
