import { normalize } from '@angular-devkit/core';
import { strings } from '@angular-devkit/core';
import { apply, move, Rule, template, url, mergeWith, chain, Tree } from '@angular-devkit/schematics';
import { GeneratorInterface } from '../GeneratorInterface';
import { GeneratorHelper } from '../GeneratorHelper';

const ROOT_REDUCER_FILE_PATH = 'src/app/store/reducers.ts';
const ROOT_SAGA_FILE_PATH = 'src/app/store/sagas.ts';
const MAIN_MODULE_FILE_PATH = 'src/app/components/components.module.ts';

const getComponentPath = function(options: any) {
  const component = options['component'] === undefined ? '' : options['component'];
  return `${options.sourceDir}/app/components/${component}`;
};

export class SingleGenerator implements GeneratorInterface {
  addDeclarationToReducer(options: any): Rule {
    return (tree: Tree) => {
      let declarationRecorder, readVal;
      if (options.component === undefined) {
        readVal = tree.read(ROOT_REDUCER_FILE_PATH);
        declarationRecorder = tree.beginUpdate(ROOT_REDUCER_FILE_PATH);
        if (readVal) {
          let val = readVal || '';
          if (val) {
            let str = val.toString();
            if (str.indexOf('const RootReducer = (') > -1) {
              declarationRecorder.insertLeft(
                0,
                `import { ${new GeneratorHelper().reducerGenerator.bind(options)()} } from '../components/${strings.dasherize(options.name)}/${strings.dasherize(
                  options.name
                )}.reducer';\n`
              );
            }
            if (str.indexOf('combineReducers({') > -1) {
              declarationRecorder.insertRight(str.indexOf('combineReducers({') + 'combineReducers({'.length, `${new GeneratorHelper().reducerGenerator.bind(options)()}, `);
            }
          }
        }
      } else {
        const component_reducer_path = `${getComponentPath(options)}/${strings.dasherize(options.component.split('/')[options.component.split('/').length - 1])}.reducer.ts`;
        console.log(component_reducer_path);
        readVal = tree.read(component_reducer_path);
        declarationRecorder = tree.beginUpdate(component_reducer_path);
        if (readVal) {
          let val = readVal || '';
          if (val) {
            let str = val.toString();
            declarationRecorder.insertLeft(
              0,
              `import { ${new GeneratorHelper().reducerGenerator.bind(options)()} } from './${strings.dasherize(options.name)}/${strings.dasherize(options.name)}.reducer';\n`
            );
            if (str.indexOf('combineReducers({') > -1) {
              declarationRecorder.insertRight(str.indexOf('combineReducers({') + 'combineReducers({'.length, `${new GeneratorHelper().reducerGenerator.bind(options)()}, `);
            }
          }
        }
      }
      tree.commitUpdate(declarationRecorder);
    };
  }

  addDeclarationToSagas(options: any): Rule {
    return (tree: Tree) => {
      let declarationRecorder, readVal;
      if (options.component === undefined) {
        readVal = tree.read(ROOT_SAGA_FILE_PATH);
        declarationRecorder = tree.beginUpdate(ROOT_SAGA_FILE_PATH);
        if (readVal) {
          let val = readVal || '';
          if (val) {
            let str = val.toString();
            declarationRecorder.insertLeft(
              0,
              `import ${new GeneratorHelper().reducerGenerator.bind(options)()} from '../components/${strings.dasherize(options.name)}/${strings.dasherize(options.name)}.saga';\n`
            );
            if (str.indexOf('yield [') > -1) {
              declarationRecorder.insertRight(str.indexOf('yield [') + 'yield ['.length, `...${new GeneratorHelper().reducerGenerator.bind(options)()}, `);
            }
          }
        }
      } else {
        const component_saga_path = `${getComponentPath(options)}/${strings.dasherize(options.component.split('/')[options.component.split('/').length - 1])}.saga.ts`;
        readVal = tree.read(component_saga_path);
        declarationRecorder = tree.beginUpdate(component_saga_path);
        if (readVal) {
          let val = readVal || '';
          if (val) {
            let str = val.toString();
            declarationRecorder.insertLeft(
              0,
              `import ${new GeneratorHelper().reducerGenerator.bind(options)()} from './${strings.dasherize(options.name)}/${strings.dasherize(options.name)}.saga';\n`
            );
            if (str.indexOf('export default [') > -1) {
              declarationRecorder.insertRight(str.indexOf('export default [') + 'export default ['.length, `...${new GeneratorHelper().reducerGenerator.bind(options)()}, `);
            }
          }
        }
      }

      tree.commitUpdate(declarationRecorder);
    };
  }
  addDeclarationToModule(options: any): Rule {
    return (tree: Tree) => {
      let declarationRecorder, readVal;
      if (options.component === undefined) {
        readVal = tree.read(MAIN_MODULE_FILE_PATH);
        declarationRecorder = tree.beginUpdate(MAIN_MODULE_FILE_PATH);
        if (readVal) {
          let val = readVal || '';
          if (val) {
            let str = val.toString();
            declarationRecorder.insertLeft(
              0,
              `import { ${strings.classify(options.name)}Component } from './${strings.dasherize(options.name)}/${strings.dasherize(options.name)}.component';\n`
            );
            if (str.indexOf('declarations: [') > -1) {
              declarationRecorder.insertRight(str.indexOf('declarations: [') + 'declarations: ['.length, `${strings.classify(options.name)}Component, `);
            }
          }
        }
      } else {
        const parent_module_path = `${getComponentPath(options)}/${strings.dasherize(options.component.split('/')[options.component.split('/').length - 1])}.module.ts`;
        readVal = tree.read(parent_module_path);
        declarationRecorder = tree.beginUpdate(parent_module_path);
        if (readVal) {
          let val = readVal || '';
          if (val) {
            let str = val.toString();
            declarationRecorder.insertLeft(
              0,
              `import { ${strings.classify(options.name)}Component } from './${strings.dasherize(options.name)}/${strings.dasherize(options.name)}.component';\n`
            );
            if (str.indexOf('declarations: [') > -1) {
              declarationRecorder.insertRight(str.indexOf('declarations: [') + 'declarations: ['.length, `${strings.classify(options.name)}Component, `);
            }
          }
        }
      }

      tree.commitUpdate(declarationRecorder);
    };
  }

  generate(options: any): Rule {
    options.path = options.path ? normalize(options.path) : options.path;

    let data = {
      ...strings,
      ...options
    };

    data = {
      ...data,
      ...{
        resolveRelativePath: new GeneratorHelper().resolveRelativePath.bind(data),
        generateFilePath: new GeneratorHelper().generateFilePath.bind(data),
        resolveApiModule: new GeneratorHelper().resolveApiModule.bind(data),
        resolveModelName: new GeneratorHelper().resolveModelName.bind(data),
        generateFilePathForApi: new GeneratorHelper().generateFilePathForApi.bind(data),
        actionGenerator: new GeneratorHelper().actionGenerator.bind(data),
        pluralizeName: new GeneratorHelper().pluralizeName.bind(data),
        reducerGenerator: new GeneratorHelper().reducerGenerator.bind(data),
        fullReducerGenerator: new GeneratorHelper().fullReducerGenerator.bind(data),
        routerGenerator: new GeneratorHelper().routerGenerator.bind(data)
      }
    };

    const merge = [];
    const declaretion = [];

    if (options['api'] === undefined) {
      throw new Error('api option is required to generate single component');
    }
    merge.push(apply(url('./files/single'), [template(data), move(getComponentPath(options))]));
    declaretion.push(this.addDeclarationToModule(options));
    if (options.noredux !== true) {
      declaretion.push(this.addDeclarationToReducer(options));
      declaretion.push(this.addDeclarationToSagas(options));
    }
    return chain([...merge.map(item => mergeWith(item)), ...declaretion]);
  }
}
